package com.ihaoin.hooloo.device.view;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.bumptech.glide.Glide;
import com.ihaoin.hooloo.device.R;
import com.ihaoin.hooloo.device.data.Goods;
import com.ihaoin.hooloo.device.util.CollectionUtils;
import com.ihaoin.hooloo.device.util.StringUtils;
import com.ihaoin.hooloo.device.util.Utils;

public class GoodsDetailDialog extends Dialog {
    private Goods goods;
    private View butnClose;
    private ImageView imgIntro;
    private TextView txtName;
    private LinearLayout layoutSpecs;
    private LinearLayout layoutImages;
    private TextView txtPrice;
    private TextView txtDiscount;
    private Button butnSubtract;
    private TextView txtCount;
    private Button butnAdd;
    private Button butnBuy;
    private Button butnTrolley;

    private Integer count;

    public GoodsDetailDialog(@NonNull Context context, Goods goods) {
        super(context);
        this.goods = goods;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.setContentView(R.layout.view_goods_detail);
        initViews();
    }

    private void initViews() {
        butnClose = findViewById(R.id.butn_close);
        imgIntro = findViewById(R.id.img_intro);
        txtName = findViewById(R.id.txt_name);
        layoutSpecs = findViewById(R.id.layout_specs);
        layoutImages = findViewById(R.id.layout_images);
        txtPrice = findViewById(R.id.txt_price);
        txtDiscount = findViewById(R.id.txt_discount);
        butnSubtract = findViewById(R.id.butn_subtract);
        txtCount = findViewById(R.id.txt_count);
        butnAdd = findViewById(R.id.butn_add);
        butnBuy = findViewById(R.id.butn_buy);
        butnTrolley = findViewById(R.id.butn_trolley);

        butnClose.setOnClickListener(x -> dismiss());

        Glide.with(getContext()).load(goods.getPics().getIntroImages().get(0)).into(imgIntro);
        txtName.setText(goods.getName());

        buildSpecs();
        buildDetails();
    }

    private void buildSpecs() {
        if (CollectionUtils.isEmpty(goods.getSpecs())) {
            return;
        }
        goods.getSpecs().forEach(spec -> {
            if (CollectionUtils.isEmpty(spec.getRules())) {
                return;
            }
            View specView = getLayoutInflater().inflate(R.layout.item_spec, null);
            TextView txtName = specView.findViewById(R.id.txt_name);
            RadioGroup specGroup = specView.findViewById(R.id.group_spec);

            txtName.setText(spec.getSpecName());
            spec.getRules().forEach(rule -> {
                RadioGroup.LayoutParams layoutParams = new RadioGroup.LayoutParams(RadioGroup.LayoutParams.WRAP_CONTENT, RadioGroup.LayoutParams.WRAP_CONTENT);
                layoutParams.rightMargin = Utils.dpToPx(getContext(), 5);
                RadioButton button = (RadioButton) getLayoutInflater().inflate(R.layout.item_spec_rule, null);
                button.setId(rule.getRuleId());
                button.setText(rule.getRuleName());
                if (rule.getIsDefault() != null && rule.getIsDefault().equals(1)) {
                    button.setChecked(true);
                }
                specGroup.addView(button, layoutParams);
            });
            layoutSpecs.addView(specView);
        });
    }

    private void buildDetails() {
        if (goods.getPics() == null || CollectionUtils.isEmpty(goods.getPics().getDetailImages())) {
            return;
        }
        goods.getPics().getDetailImages().forEach(url -> {
            if (StringUtils.isEmpty(url)) {
                return;
            }
            ImageView imageView = new ImageView(getContext());
            imageView.setAdjustViewBounds(true);
            LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
            layoutParams.bottomMargin = Utils.dpToPx(getContext(), 5);
            Glide.with(getContext()).load(url).into(imageView);
            layoutImages.addView(imageView, layoutParams);
        });
    }
}
